// ex3.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <windows.h>
#include <conio.h>
#include <assert.h>
#include <strsafe.h>


typedef struct _ProcessTimes {
    FILETIME CreateTime,    // time of process creation
        ExitTime,      // time of process termination
        KernelTime,    // amount of time spent in kernel mode
        UserTime;      // amount of time spent in user mode
} PROCESSTIMES,
*PPROCESSTIMES;


BOOL PrintProcessTimes(LPCTSTR proc_name, PPROCESSTIMES pt)
{
    ULARGE_INTEGER uCreateTime = {pt->CreateTime.dwLowDateTime, pt->CreateTime.dwHighDateTime};
    ULARGE_INTEGER uExitTime = {pt->ExitTime.dwLowDateTime, pt->ExitTime.dwHighDateTime};
    ULARGE_INTEGER uUserTime = {pt->UserTime.dwLowDateTime, pt->UserTime.dwHighDateTime};
    ULARGE_INTEGER uKernelTime = {pt->KernelTime.dwLowDateTime, pt->KernelTime.dwHighDateTime};

    ULONGLONG runTime = uExitTime.QuadPart - uCreateTime.QuadPart;
    ULONGLONG cpuTime = uUserTime.QuadPart + uKernelTime.QuadPart;

    _tprintf(_T("Process %s:\n\tTotal run time: %d\n")
             _T("\tTotal CPU time: %d\n")
             _T("\tTime in Kernel mode: %d\n")
             _T("\tTime in user mode: %d\n"),
                proc_name, DWORD(runTime/10), DWORD(cpuTime/10),
                DWORD(uKernelTime.QuadPart/10), DWORD(uUserTime.QuadPart/10));
 
    return TRUE;
}


BOOL CreateBeeperProcess(DWORD freq, DWORD delta, LPPROCESS_INFORMATION lpPi)
{
    assert(lpPi != NULL);

    //prepare command line arguments
    TCHAR cmd_line[MAX_PATH];
    StringCchPrintf(cmd_line, MAX_PATH, _T("beeper.exe %d %d"), freq, delta); 

    // intialize STARTUPINFO structure
    STARTUPINFO si;
    ZeroMemory(&si, sizeof(si));

    // intialize PROCESS_INFORMATION structure
    ZeroMemory(lpPi, sizeof(PROCESS_INFORMATION));

    // create the new process
    return CreateProcess(NULL,
                         cmd_line,
                         NULL,
                         NULL,
                         FALSE,
                         0,
                         NULL,
                         NULL,
                         &si,
                         lpPi);
}


int _tmain(int argc, _TCHAR* argv[])
{
    PROCESS_INFORMATION pi1, pi2;

    BOOL bRes1 = CreateBeeperProcess(500,  1000, &pi1);
    BOOL bRes2 = CreateBeeperProcess(1000, 1733, &pi2);
    assert(bRes1  &&  bRes2);

    _tprintf(_T("processes started, press any key to stop\n"));
    _getch();

    //kill processes, not very nice 
    BOOL bTermRes1 = TerminateProcess(pi1.hProcess, 0);
    BOOL bTermRes2 = TerminateProcess(pi2.hProcess, 0);
    assert(bTermRes1  &&  bTermRes2);

    _tprintf(_T("processes terminated\n"));
    _getch();

    // get process statistics
    PROCESSTIMES pt1, pt2;
    bRes1 = GetProcessTimes(pi1.hProcess,
                            &pt1.CreateTime, &pt1.ExitTime,
                            &pt1.KernelTime, &pt1.UserTime);
    bRes2 = GetProcessTimes(pi2.hProcess,
                            &pt2.CreateTime, &pt2.ExitTime,
                            &pt2.KernelTime, &pt2.UserTime);
    assert(bRes1  &&  bRes2);

    // close all handles
    CloseHandle(pi1.hProcess);
    CloseHandle(pi2.hProcess);
    CloseHandle(pi1.hThread);
    CloseHandle(pi2.hThread);

    //print process times
    BOOL bPrintRes1 = PrintProcessTimes(_T("A"), &pt1);
    BOOL bPrintRes2 = PrintProcessTimes(_T("B"), &pt2);

    return 0;
}

